@@ -12,10 +12,13 @@ use std::str;
 
 use anyhow::Result;
 use bio::stats::bayesian::bayes_factors::evidence::KassRaftery;
-use bio::stats::LogProb;
+use bio::stats::{LogProb, PHREDProb, Prob};
+use bio_types::genome::{self, AbstractLocus};
 use bio_types::sequence::SequenceReadPairOrientation;
 use counter::Counter;
+use ordered_float::NotNan;
 use rust_htslib::bam;
+use statrs::statistics::OrderStatistics;
 
 use serde::Serialize;
 // use bio::stats::bayesian::bayes_factors::evidence::KassRaftery;
@@ -24,8 +27,8 @@ use itertools::Itertools;
 
 use crate::errors::{self, Error};
 use crate::estimation::alignment_properties::AlignmentProperties;
-use crate::utils;
 use crate::utils::homopolymers::HomopolymerErrorModel;
+use crate::utils::{self, PROB_05};
 use crate::utils::{PROB_09, PROB_095};
 use crate::variants::sample;
 use crate::variants::types::Variant;
@@ -34,7 +37,7 @@ use super::realignment::pairhmm::RefBaseVariantEmission;
 use super::realignment::Realignable;
 
 /// Calculate expected value of sequencing depth, considering mapping quality.
-pub(crate) fn expected_depth(obs: &[Observation<ReadPosition>]) -> u32 {
+pub(crate) fn expected_depth(obs: &[ProcessedObservation]) -> u32 {
     LogProb::ln_sum_exp(&obs.iter().map(|o| o.prob_mapping).collect_vec())
         .exp()
         .round() as u32
@@ -162,11 +165,70 @@ pub(crate) fn read_orientation(record: &bam::Record) -> Result<SequenceReadPairO
     }
 }
 
+#[derive(Debug, Clone, Derefable, Default)]
+pub(crate) struct ExactAltLoci {
+    #[deref]
+    inner: Vec<genome::Locus>,
+}
+
+impl<'a> From<&'a bam::Record> for ExactAltLoci {
+    fn from(record: &'a bam::Record) -> Self {
+        match record.aux(b"XA") {
+            Ok(bam::record::Aux::String(xa)) => {
+                ExactAltLoci {
+                    inner: xa
+                        .split(';')
+                        .filter_map(|xa| {
+                            if xa.is_empty() {
+                                // last semicolon passed
+                                None
+                            } else {
+                                let items: Vec<_> = xa.split(',').collect();
+                                if items.len() == 4 {
+                                    let contig = items[0];
+                                    let mut pos = items[1];
+                                    if pos.starts_with('-') || pos.starts_with('-') {
+                                        pos = &pos[1..];
+                                    }
+                                    if let Ok(pos) = pos.parse() {
+                                        Some(genome::Locus::new(contig.to_owned(), pos))
+                                    } else {
+                                        None
+                                    }
+                                } else {
+                                    warn!("{}", INVALID_XA_FORMAT_MSG);
+                                    None
+                                }
+                            }
+                        })
+                        .collect(),
+                }
+            }
+            Ok(tag) => {
+                warn!("{}", INVALID_XA_FORMAT_MSG);
+                ExactAltLoci::default()
+            }
+            Err(e) => {
+                // no XA tag found, return empty.
+                ExactAltLoci::default()
+            }
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
+pub(crate) enum AltLocus {
+    Major,
+    Some,
+    None,
+}
+
 /// An observation for or against a variant.
 #[derive(Clone, Debug, Builder, Default, Serialize)]
-pub(crate) struct Observation<P = Option<u32>>
+pub(crate) struct Observation<P = Option<u32>, A = ExactAltLoci>
 where
     P: Clone,
+    A: Clone,
 {
     name: Option<String>,
     /// Posterior probability that the read/read-pair has been mapped correctly (1 - MAPQ).
@@ -207,11 +269,16 @@ where
     pub(crate) read_position: P,
     /// Probability to make this observation at a homopolymer artifact
     pub(crate) prob_observable_at_homopolymer_artifact: Option<LogProb>,
+    pub(crate) prob_observable_at_homopolymer_variant: Option<LogProb>,
     /// Homopolymer indel length (None if there is no homopolymer indel compared to reference)
     pub(crate) homopolymer_indel_len: Option<i8>,
+    pub(crate) is_max_mapq: bool,
+    pub(crate) alt_locus: A,
 }
 
-impl<P: Clone> ObservationBuilder<P> {
+pub(crate) type ProcessedObservation = Observation<ReadPosition, AltLocus>;
+
+impl<P: Clone, A: Clone> ObservationBuilder<P, A> {
     pub(crate) fn prob_mapping_mismapping(&mut self, prob_mapping: LogProb) -> &mut Self {
         self.prob_mapping(prob_mapping)
             .prob_mismapping(prob_mapping.ln_one_minus_exp())
@@ -223,8 +290,13 @@ impl<P: Clone> ObservationBuilder<P> {
     }
 }
 
-impl Observation<Option<u32>> {
-    pub(crate) fn process(&self, major_read_position: Option<u32>) -> Observation<ReadPosition> {
+impl Observation<Option<u32>, ExactAltLoci> {
+    pub(crate) fn process(
+        &self,
+        major_read_position: Option<u32>,
+        major_alt_locus: &Option<genome::Locus>,
+        alignment_properties: &AlignmentProperties,
+    ) -> Observation<ReadPosition, AltLocus> {
         Observation {
             name: self.name.clone(),
             prob_mapping: self.prob_mapping,
@@ -254,12 +326,29 @@ impl Observation<Option<u32>> {
                 }
             }),
             prob_observable_at_homopolymer_artifact: self.prob_observable_at_homopolymer_artifact,
+            prob_observable_at_homopolymer_variant: self.prob_observable_at_homopolymer_variant,
             homopolymer_indel_len: self.homopolymer_indel_len,
+            is_max_mapq: self.is_max_mapq,
+            alt_locus: if let Some(major_alt_locus) = major_alt_locus {
+                if self.alt_locus.iter().any(|alt_locus| {
+                    locus_to_bucket(alt_locus, alignment_properties) == *major_alt_locus
+                }) {
+                    AltLocus::Major
+                } else {
+                    if self.alt_locus.is_empty() {
+                        AltLocus::None
+                    } else {
+                        AltLocus::Some
+                    }
+                }
+            } else {
+                AltLocus::None
+            },
         }
     }
 }
 
-impl<P: Clone> Observation<P> {
+impl<P: Clone, A: Clone> Observation<P, A> {
     pub(crate) fn bayes_factor_alt(&self) -> BayesFactor {
         BayesFactor::new(self.prob_alt, self.prob_ref)
     }
@@ -300,15 +389,13 @@ impl<P: Clone> Observation<P> {
     }
 
     pub(crate) fn is_strong_alt_support(&self) -> bool {
-        self.is_uniquely_mapping()
-            && BayesFactor::new(self.prob_alt, self.prob_ref).evidence_kass_raftery()
-                >= KassRaftery::Strong
+        BayesFactor::new(self.prob_alt, self.prob_ref).evidence_kass_raftery()
+            >= KassRaftery::Strong
     }
 
     pub(crate) fn is_strong_ref_support(&self) -> bool {
-        self.is_uniquely_mapping()
-            && BayesFactor::new(self.prob_ref, self.prob_alt).evidence_kass_raftery()
-                >= KassRaftery::Strong
+        BayesFactor::new(self.prob_ref, self.prob_alt).evidence_kass_raftery()
+            >= KassRaftery::Strong
     }
 
     pub(crate) fn is_ref_support(&self) -> bool {
@@ -320,41 +407,6 @@ impl<P: Clone> Observation<P> {
             >= KassRaftery::Positive
     }
 
-    pub(crate) fn adjust_prob_mapping(pileup: &mut [Self]) {
-        if !pileup.is_empty() {
-            // METHOD: adjust MAPQ to get rid of stochastically inflated ones
-            // This takes the arithmetic mean of all MAPQs in the pileup.
-            // By that, we effectively diminish high MAPQs of reads that just achieve them
-            // because of e.g. randomly better matching bases in themselves or their mates.
-            let mut prob_sum = LogProb::ln_sum_exp(
-                &pileup
-                    .iter()
-                    .map(|obs| obs.prob_mapping_orig())
-                    .collect_vec(),
-            );
-
-            let calc_adjusted = |prob_sum: LogProb, n| LogProb(*prob_sum - (n as f64).ln());
-            let mut adjusted = calc_adjusted(prob_sum, pileup.len());
-
-            if pileup.len() < 20 {
-                // METHOD: for low depths, this method does not reliably work because it can be that by accident the
-                // low MAPQ reads are not in the pileup. In order to correct for this sampling issue,
-                // we add one pseudo low MAPQ observation. The higher the depth becomes, the less this observation
-                // plays a role.
-                prob_sum = prob_sum.ln_add_exp(adjusted + *PROB_09);
-
-                adjusted = calc_adjusted(prob_sum, pileup.len() + 1);
-            }
-
-            for obs in pileup {
-                if adjusted < obs.prob_mapping_orig() {
-                    obs.prob_mapping_adj = Some(adjusted);
-                    obs.prob_mismapping_adj = Some(adjusted.ln_one_minus_exp());
-                }
-            }
-        }
-    }
-
     pub(crate) fn has_homopolymer_error(&self) -> bool {
         self.homopolymer_indel_len
             .map(|indel_len| indel_len != 0)
@@ -362,7 +414,59 @@ impl<P: Clone> Observation<P> {
     }
 }
 
-pub(crate) fn major_read_position(pileup: &[Observation<Option<u32>>]) -> Option<u32> {
+impl ProcessedObservation {
+    pub(crate) fn adjust_prob_mapping(
+        pileup: &mut [Self],
+        alignment_properties: &AlignmentProperties,
+    ) {
+        if !pileup.is_empty() {
+            // METHOD: adjust MAPQ to get rid of stochastically inflated ones.
+            // The idea here is to estimate the probability that this locus is
+            // actually filled with reads from a homolog (e.g. induced by another variant somewhere else), say ph.
+            // We do this by considering all non-max MAPQs as indicative of this.
+            // Conservatively, we recalibrate those to a probability of 0.5.
+            // Then, adjusted MAPQs are calculated as ph * 0.5 + 1-ph * max_mapq.
+            // Technically, we just compute the mean here, which yields the same result.
+
+            let max_prob_mapping =
+                LogProb::from(PHREDProb(alignment_properties.max_mapq as f64)).ln_one_minus_exp();
+
+            let probs = pileup
+                .iter()
+                .map(|obs| {
+                    if relative_eq!(*obs.prob_mapping_orig(), *max_prob_mapping) {
+                        obs.prob_mapping_orig()
+                    } else {
+                        *PROB_05
+                    }
+                })
+                .collect_vec();
+
+            let mut prob_sum = LogProb::ln_sum_exp(&probs);
+
+            let calc_average = |prob_sum: LogProb, n| LogProb(*prob_sum - (n as f64).ln());
+            let mut average = calc_average(prob_sum, pileup.len());
+            if pileup.len() < 20 {
+                // METHOD: for low depths, this method does not reliably work because it can be that by accident the
+                // low MAPQ reads are not in the pileup. In order to correct for this sampling issue,
+                // we add one pseudo low MAPQ observation. The higher the depth becomes, the less this observation
+                // plays a role.
+                prob_sum = prob_sum.ln_add_exp(*PROB_05);
+
+                average = calc_average(prob_sum, pileup.len() + 1);
+            }
+
+            for obs in pileup {
+                obs.prob_mapping_adj = Some(average);
+                obs.prob_mismapping_adj = Some(average.ln_one_minus_exp());
+            }
+        }
+    }
+}
+
+pub(crate) fn major_read_position(
+    pileup: &[Observation<Option<u32>, ExactAltLoci>],
+) -> Option<u32> {
     let counter: Counter<_> = pileup.iter().filter_map(|obs| obs.read_position).collect();
     let most_common = counter.most_common();
     if most_common.is_empty() {
@@ -372,6 +476,41 @@ pub(crate) fn major_read_position(pileup: &[Observation<Option<u32>>]) -> Option
     }
 }
 
+pub(crate) fn major_alt_locus(
+    pileup: &[Observation<Option<u32>, ExactAltLoci>],
+    alignment_properties: &AlignmentProperties,
+) -> Option<genome::Locus> {
+    let counter: Counter<_> = pileup
+        .iter()
+        .map(|obs| {
+            obs.alt_locus
+                .iter()
+                .map(|locus| locus_to_bucket(locus, alignment_properties))
+        })
+        .flatten()
+        .collect();
+    let most_common = counter.most_common();
+    if most_common.is_empty() {
+        None
+    } else {
+        Some(most_common[0].0.clone())
+    }
+}
+
+pub(crate) fn locus_to_bucket(
+    locus: &genome::Locus,
+    alignment_properties: &AlignmentProperties,
+) -> genome::Locus {
+    // METHOD: map each locus to the nearest multiple of the read len from the left.
+    // This way, varying reads become comparable
+
+    let coeff = alignment_properties.max_read_len as u64 * 10;
+
+    let bucket = genome::Locus::new(locus.contig().to_owned(), (locus.pos() / coeff) * coeff);
+
+    bucket
+}
+
 /// Something that can be converted into observations.
 pub(crate) trait Observable<E>: Variant<Evidence = E>
 where
@@ -389,6 +528,9 @@ where
     /// correctly.
     fn prob_mapping(&self, evidence: &E) -> LogProb;
 
+    /// Return the minimum MAPQ of all records involved in the given evidence.
+    fn min_mapq(&self, evidence: &E) -> u8;
+
     /// Calculate an observation from the given evidence.
     fn evidence_to_observation(
         &self,
@@ -403,7 +545,7 @@ where
                 // Unstranded observations (e.g. only insert size), are too unreliable, or do not contain
                 // any information (e.g. no overlap).
                 Some(allele_support) if allele_support.strand() != Strand::None => {
-                    let read_indel_len = allele_support.homopolymer_indel_len().unwrap_or(0);
+                    let alt_indel_len = allele_support.homopolymer_indel_len().unwrap_or(0);
 
                     let mut obs = ObservationBuilder::default();
                     obs.name(Some(str::from_utf8(evidence.name()).unwrap().to_owned()))
@@ -422,28 +564,54 @@ where
                         .softclipped(evidence.softclipped())
                         .read_position(allele_support.read_position())
                         .paired(evidence.is_paired())
-                        .prob_hit_base(LogProb::ln_one() - LogProb((evidence.len() as f64).ln()));
+                        .prob_hit_base(LogProb::ln_one() - LogProb((evidence.len() as f64).ln()))
+                        .is_max_mapq(self.min_mapq(evidence) == alignment_properties.max_mapq)
+                        .alt_locus(evidence.alt_loci());
 
                     if let Some(homopolymer_error_model) = homopolymer_error_model {
-                        let ref_indel_len = read_indel_len
-                            + homopolymer_error_model.variant_homopolymer_indel_len();
+                        let ref_indel_len =
+                            alt_indel_len + homopolymer_error_model.variant_homopolymer_indel_len();
                         if ref_indel_len == 0 {
                             // no homopolymer indel in read compared to reference
                             obs.homopolymer_indel_len(None)
-                                .prob_observable_at_homopolymer_artifact(None);
+                                .prob_observable_at_homopolymer_artifact(None)
+                                .prob_observable_at_homopolymer_variant(None);
                         } else {
-                            obs.homopolymer_indel_len(Some(read_indel_len))
-                                .prob_observable_at_homopolymer_artifact(if ref_indel_len == 0 {
-                                    unreachable!("caught above");
-                                } else if ref_indel_len > 0 {
-                                    Some(homopolymer_error_model.prob_homopolymer_insertion())
+                            obs.homopolymer_indel_len(Some(alt_indel_len));
+                            assert!(ref_indel_len != 0); // caught above
+                            if ref_indel_len > 0 {
+                                obs.prob_observable_at_homopolymer_artifact(Some(
+                                    homopolymer_error_model.prob_homopolymer_artifact_insertion(),
+                                ))
+                                .prob_observable_at_homopolymer_variant(if alt_indel_len == 0 {
+                                    Some(
+                                        homopolymer_error_model.prob_homopolymer_variant_no_indel(),
+                                    )
                                 } else {
-                                    Some(homopolymer_error_model.prob_homopolymer_deletion())
+                                    Some(
+                                        homopolymer_error_model
+                                            .prob_homopolymer_variant_insertion(),
+                                    )
                                 });
+                            } else {
+                                obs.prob_observable_at_homopolymer_artifact(Some(
+                                    homopolymer_error_model.prob_homopolymer_artifact_deletion(),
+                                ))
+                                .prob_observable_at_homopolymer_variant(if alt_indel_len == 0 {
+                                    Some(
+                                        homopolymer_error_model.prob_homopolymer_variant_no_indel(),
+                                    )
+                                } else {
+                                    Some(
+                                        homopolymer_error_model.prob_homopolymer_variant_deletion(),
+                                    )
+                                });
+                            }
                         }
                     } else {
                         obs.homopolymer_indel_len(None)
-                            .prob_observable_at_homopolymer_artifact(None);
+                            .prob_observable_at_homopolymer_artifact(None)
+                            .prob_observable_at_homopolymer_variant(None);
                     }
 
                     Some(obs.build().unwrap())
@@ -455,6 +623,8 @@ where
 }
 
 pub(crate) trait Evidence {
+    fn alt_loci(&self) -> ExactAltLoci;
+
     fn read_orientation(&self) -> Result<SequenceReadPairOrientation>;
 
     fn softclipped(&self) -> bool;
@@ -479,6 +649,8 @@ impl Deref for SingleEndEvidence {
     }
 }
 
+const INVALID_XA_FORMAT_MSG: &'static str = "XA tag of bam records in unexpected format. Expecting string (type Z) in bwa format (chr,pos,CIGAR,NM;).";
+
 impl Evidence for SingleEndEvidence {
     fn read_orientation(&self) -> Result<SequenceReadPairOrientation> {
         // Single end evidence can just mean that we only need to consider each read alone,
@@ -502,6 +674,10 @@ impl Evidence for SingleEndEvidence {
     fn name(&self) -> &[u8] {
         self.inner.qname()
     }
+
+    fn alt_loci(&self) -> ExactAltLoci {
+        ExactAltLoci::from(self.inner.as_ref())
+    }
 }
 
 impl PartialEq for SingleEndEvidence {
@@ -570,6 +746,17 @@ impl Evidence for PairedEndEvidence {
             PairedEndEvidence::SingleEnd(rec) => rec.qname(),
         }
     }
+
+    fn alt_loci(&self) -> ExactAltLoci {
+        match self {
+            PairedEndEvidence::SingleEnd(rec) => ExactAltLoci::from(rec.as_ref()),
+            PairedEndEvidence::PairedEnd { left, right } => {
+                let mut left = ExactAltLoci::from(left.as_ref());
+                left.inner.extend(ExactAltLoci::from(right.as_ref()).inner);
+                left
+            }
+        }
+    }
 }
 
 impl PartialEq for PairedEndEvidence {
